@@ -72,6 +72,9 @@ class MainWindow(QMainWindow):
     # only emitted when this is the last MainWindow to close
     aboutToCloseLast = pyqtSignal()
 
+    # emitted when all editor documents have been closed
+    allDocumentsClosed = pyqtSignal()
+
     # both signals emit (current, previous)
     currentDocumentChanged = pyqtSignal(document.Document, document.Document)
     currentViewChanged = pyqtSignal(view.View, view.View)
@@ -493,9 +496,10 @@ class MainWindow(QMainWindow):
 
     def openDocument(self):
         """ Displays an open dialog to open one or more documents. """
-        if app.documents:
-            ext = os.path.splitext(self.currentDocument().url().path())[1]
-            directory = os.path.dirname(self.currentDocument().url().toLocalFile()) or app.basedir()
+        d = self.currentDocument()
+        if d:
+            ext = os.path.splitext(d.url().path())[1]
+            directory = os.path.dirname(d.url().toLocalFile()) or app.basedir()
         else:
             ext = ".ly"
             directory = app.basedir()
@@ -519,7 +523,14 @@ class MainWindow(QMainWindow):
             if filename:
                 filetypes = app.filetypes(os.path.splitext(filename)[1])
             else:
-                directory = app.basedir() # default directory to save to
+                # find a suitable directory to save to
+                for d in self.historyManager.documents()[1::]:
+                    if d.url().toLocalFile():
+                        directory = os.path.dirname(d.url().toLocalFile())
+                        break
+                else:
+                    directory = app.basedir() # default directory to save to
+                
                 import documentinfo
                 import ly.lex
                 filename = os.path.join(directory, documentinfo.defaultfilename(doc))
@@ -689,6 +700,7 @@ class MainWindow(QMainWindow):
         sessions.manager.get(self).saveCurrentSessionIfDesired()
         if self.queryClose():
             sessions.setCurrentSession(None)
+            self.allDocumentsClosed.emit()
             self.cleanStart()
 
     def quit(self):
