@@ -105,6 +105,7 @@ class MusicViewPanel(panel.Panel):
         ac.music_jump_to_cursor.triggered.connect(self.jumpToCursor)
         ac.music_sync_cursor.triggered.connect(self.toggleSyncCursor)
         ac.music_copy_image.triggered.connect(self.copyImage)
+        ac.music_copy_text.triggered.connect(self.copyText)
         ac.music_document_select.documentsChanged.connect(self.updateActions)
         ac.music_copy_image.setEnabled(False)
         ac.music_next_page.triggered.connect(self.slotNextPage)
@@ -116,7 +117,15 @@ class MusicViewPanel(panel.Panel):
         ac.music_reload.triggered.connect(self.reloadView)
         self.actionCollection.music_sync_cursor.setChecked(
             QSettings().value("musicview/sync_cursor", False, bool))
-                
+        
+        mode = QSettings().value("muziekview/layoutmode", "single", type(""))
+        if mode == "double_left":
+            ac.music_two_pages_first_left.setChecked(True)
+        elif mode == "double_right":
+            ac.music_two_pages_first_right.setChecked(True)
+        else: # mode == "single":
+            ac.music_single_pages.setChecked(True)
+        
     def translateUI(self):
         self.setWindowTitle(_("window title", "Music View"))
         self.toggleViewAction().setText(_("&Music View"))
@@ -127,8 +136,18 @@ class MusicViewPanel(panel.Panel):
         w.zoomChanged.connect(self.slotMusicZoomChanged)
         w.updateZoomInfo()
         w.view.surface().selectionChanged.connect(self.updateSelection)
-        w.view.surface().pageLayout().setPagesPerRow(1)   # default to single
-        w.view.surface().pageLayout().setPagesFirstRow(0) # pages
+        
+        # read layout mode setting before using the widget
+        layout = w.view.surface().pageLayout()
+        if self.actionCollection.music_two_pages_first_right.isChecked():
+            layout.setPagesPerRow(2)
+            layout.setPagesFirstRow(1)
+        elif self.actionCollection.music_two_pages_first_left.isChecked():
+            layout.setPagesPerRow(2)
+            layout.setPagesFirstRow(0)
+        else: # "single"
+            layout.setPagesPerRow(1)   # default to single
+            layout.setPagesFirstRow(0) # pages
         
         import qpopplerview.pager
         self._pager = p = qpopplerview.pager.Pager(w.view)
@@ -149,8 +168,34 @@ class MusicViewPanel(panel.Panel):
             QTimer.singleShot(0, open)
         return w
     
+    def setPageLayoutMode(self, mode):
+        """Change the page layout and store the setting as well.
+        
+        The mode is "single", "double_left" or "double_right".
+        
+        "single": a vertical row of single pages
+        "double_left": two pages besides each other, first page is a left page
+        "double_right": two pages, first page is a right page.
+        
+        """
+        layout = self.widget().view.surface().pageLayout()
+        if mode == "double_right":
+            layout.setPagesPerRow(2)
+            layout.setPagesFirstRow(1)
+        elif mode == "double_left":
+            layout.setPagesPerRow(2)
+            layout.setPagesFirstRow(0)
+        elif mode == "single":
+            layout.setPagesPerRow(1)
+            layout.setPagesFirstRow(0)
+        else:
+            raise ValueError("wrong mode value")
+        QSettings().setValue("muziekview/layoutmode", mode)
+        layout.update()
+    
     def updateSelection(self, rect):
         self.actionCollection.music_copy_image.setEnabled(bool(rect))
+        self.actionCollection.music_copy_text.setEnabled(bool(rect))
     
     def updatePagerLanguage(self):
         self.actionCollection.music_pager.setPageCount(self._pager.pageCount())
@@ -232,24 +277,15 @@ class MusicViewPanel(panel.Panel):
     
     @activate
     def viewSinglePages(self):
-        layout = self.widget().view.surface().pageLayout()
-        layout.setPagesPerRow(1)
-        layout.setPagesFirstRow(0)
-        layout.update()
+        self.setPageLayoutMode("single")
     
     @activate
     def viewTwoPagesFirstRight(self):
-        layout = self.widget().view.surface().pageLayout()
-        layout.setPagesPerRow(2)
-        layout.setPagesFirstRow(1)
-        layout.update()
+        self.setPageLayoutMode("double_right")
     
     @activate
     def viewTwoPagesFirstLeft(self):
-        layout = self.widget().view.surface().pageLayout()
-        layout.setPagesPerRow(2)
-        layout.setPagesFirstRow(0)
-        layout.update()
+        self.setPageLayoutMode("double_left")
     
     @activate
     def jumpToCursor(self):
@@ -268,9 +304,18 @@ class MusicViewPanel(panel.Panel):
             self.actionCollection.music_sync_cursor.isChecked())
      
     def copyImage(self):
-        from . import image
-        image.copy(self)
+        page = self.widget().view.surface().selectedPage()
+        if not page:
+            return
+        rect = self.widget().view.surface().selectedPageRect(page)
+        import copy2image
+        copy2image.copy_image(self, page, rect, documents.filename(page.document()))
         
+    def copyText(self):
+        text = self.widget().view.surface().selectedText()
+        if text:
+            QApplication.clipboard().setText(text)
+    
     def slotZoomChanged(self, mode, scale):
         """Called when the combobox is changed, changes view zoom."""
         self.activate()
@@ -308,6 +353,7 @@ class Actions(actioncollection.ActionCollection):
         self.music_jump_to_cursor = QAction(panel)
         self.music_sync_cursor = QAction(panel, checkable=True)
         self.music_copy_image = QAction(panel)
+        self.music_copy_text = QAction(panel)
         self.music_pager = PagerAction(panel)
         self.music_next_page = QAction(panel)
         self.music_prev_page = QAction(panel)
@@ -323,6 +369,7 @@ class Actions(actioncollection.ActionCollection):
         self.music_maximize.setIcon(icons.get('view-fullscreen'))
         self.music_jump_to_cursor.setIcon(icons.get('go-jump'))
         self.music_copy_image.setIcon(icons.get('edit-copy'))
+        self.music_copy_text.setIcon(icons.get('edit-copy'))
         self.music_next_page.setIcon(icons.get('go-next'))
         self.music_prev_page.setIcon(icons.get('go-previous'))
         
@@ -351,6 +398,7 @@ class Actions(actioncollection.ActionCollection):
         self.music_jump_to_cursor.setText(_("&Jump to Cursor Position"))
         self.music_sync_cursor.setText(_("S&ynchronize with Cursor Position"))
         self.music_copy_image.setText(_("Copy to &Image..."))
+        self.music_copy_text.setText(_("Copy Selected &Text"))
         self.music_next_page.setText(_("Next Page"))
         self.music_next_page.setIconText(_("Next"))
         self.music_prev_page.setText(_("Previous Page"))
